# Copyright 2009 Paul Hummer
# This file is part of Tarmac.
#
# Tarmac is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by
# the Free Software Foundation.
#
# Tarmac is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Tarmac.  If not, see <http://www.gnu.org/licenses/>.

'''Tarmac plugin for running tests on multiple proposals.'''

# Head off lint warnings.
os = None
subprocess = None

from bzrlib.lazy_import import lazy_import
lazy_import(globals(), '''
    import os
    import subprocess
    ''')

from tarmac.exceptions import TarmacMergeError
from tarmac.hooks import TarmacHookPoint, tarmac_hooks
from tarmac.plugins import TarmacPlugin


class VerifyCommandFailed(TarmacMergeError):
    """Running the verify_command failed."""


class BundleCommand(TarmacPlugin):

    def run(self, command, target, collected_proposals):
        """Plugin for running the registered bundle verify command on target.

        This is very similar to the Command plugin, but instead of running the
        verify command on target for individual proposals, it gets a list of
        proposals merged into target and then runs the verify command.
        Updating the list of proposals in case of failure.
        """
        try:
            self.verify_command = target.config.bundle_verify_command
        except AttributeError:
            # This can be killed two versions after 0.4, whatever version that
            # is.
            try:
                self.verify_command = target.config.test_command
                self.logger.warn(
                    'test_command config setting is deprecated. '
                    'Please use verify_command instead.')
            except AttributeError:
                return

        self.logger.debug('Running test command: %s' % self.verify_command)
        cwd = os.getcwd()
        os.chdir(target.tree.abspath(''))
        proc = subprocess.Popen(
            self.verify_command,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
        stdout_value, stderr_value = proc.communicate()
        return_code = proc.wait()
        os.chdir(cwd)
        self.logger.debug('Completed test command: %s' % self.verify_command)

        if return_code != 0:
            for source, proposal in collected_proposals:
                self.do_failed(proposal, stdout_value, stderr_value)
            message = u'Test command "%s" failed.' % self.verify_command
            comment = (
                u'Bundle lander failed to land proposals: %s' %
                collected_proposals)
            raise VerifyCommandFailed(message, comment)
        else:
            self.logger.info('==========')
            self.logger.info(stdout_value)

    def do_failed(self, proposal, stdout_value, stderr_value):
        '''Perform failure tests.

        In this case, the output of the test command is posted as a comment,
        and the merge proposal is then set to "Needs review" so that Tarmac
        doesn't attempt to merge it again without human interaction.
        '''
        message = u'Test command "%s" failed.' % self.verify_command
        comment = (u'The attempt to merge %(source)s into %(target)s failed. '
                   u'Below is the output from the failed tests.\n\n'
                   u'%(output)s') % {
            'source': proposal.source_branch.display_name,
            'target': proposal.target_branch.display_name,
            'output': u'\n'.join([stdout_value, stderr_value]),
            }

        self.logger.warn(
            u'Merging %(source)s into %(target)s failed: %(msg)s' %
            {'source': proposal.source_branch.display_name,
             'target': proposal.target_branch.display_name,
             'msg': str(message)})

        subject = u'Re: [Merge] %(source)s into %(target)s' % {
            "source": proposal.source_branch.display_name,
            "target": proposal.target_branch.display_name}

        proposal.createComment(subject=subject,
                               content=comment)
        proposal.setStatus(status=u'Needs review')
        proposal.lp_save()

# Create the hook here so we don't change the tarmac_hooks API with this very
# specific hook.
tarmac_hooks.create_hook(TarmacHookPoint('tarmac_bundle_merge',
    'Called by the bundle merge command.',
    (0, 3, 4), False))
tarmac_hooks['tarmac_bundle_merge'].hook(
    BundleCommand(), 'BundleCommand plugin')
